@@ -34,11 +34,11 @@ RDF::LinkedData - A Linked Data server implementation
 
 =head1 VERSION
 
-Version 1.921_01
+Version 1.921_02
 
 =cut
 
- our $VERSION = '1.921_01';
+ our $VERSION = '1.921_02';
 
 
 =head1 SYNOPSIS
@@ -479,6 +479,14 @@ sub response {
 	}
 
 	my $type = $self->type;
+
+	if (($type eq 'controls')  && (!$self->writes_enabled)) {
+	  $response->status(404);
+	  $response->headers->content_type('text/plain');
+	  $response->body('HTTP 404: Controls are only available in read-enabled applications');
+	  return $response;
+	}
+
 	$self->type('');
 	my $node = $self->my_node($uri);
 	$self->log->info("Try rendering '$type' page for subject node: " . $node->as_string);
@@ -672,7 +680,7 @@ sub _content {
 														  title => $preds->title( $node ),
 														  base => $self->base_uri,
 														  namespacemap => $self->namespaces);
-		my $writer = HTML::HTML5::Writer->new( charset => 'ascii', markup => 'html' );
+		my $writer = HTML::HTML5::Writer->new( charset => 'ascii', markup => 'html', end_tags => 1);
 		$output{body} = $writer->document($gen->create_document($returnmodel));
 		$output{content_type} = 'text/html';
 	}
@@ -813,7 +821,7 @@ sub _void_content {
 															 base => $self->base_uri,
 															 namespacemap => $self->namespaces);
 			my $markup = ($ct eq 'application/xhtml+xml') ? 'xhtml' : 'html';
-			my $writer = HTML::HTML5::Writer->new( charset => 'ascii', markup => $markup );
+			my $writer = HTML::HTML5::Writer->new( charset => 'ascii', markup => $markup, end_tags => 1 );
 			$body = $writer->document($gen->create_document($self->_voidmodel));
 		}
 		my $response = Plack::Response->new;
